function handles=m_gen_calc_dye_coords(handles)
% This function return the initial position of the contact points, the
% initial position and orientation of the dye molecules on the different
% structures and sets the "is_mol_coords_calc" vector its appropriate value.
% Only recalculates these values if they have not already been calculated
% after adding or modifying the pattern. Does not update the handles
% variable, it should be done inside its "parent" function.

% PARENT: testSTORM.m
% CHILD: m_gen_mol_coord.m

is_mol_coords_calc=handles.seq_params.labels.is_mol_coords_calc;    % vector storing values for deciding whether the positions and orientations on the patterns needs to be recalculated
% getting the acquisition parameters
params.aq=handles.seq_params.aq;
params.psf=handles.seq_params.psf;

% getting whether one or two kind of dyes are used
switch get(handles.N_of_dyes_checkbox,'Value')
    case 0
        N_of_dyes=1;
    case 1
        N_of_dyes=2;
end

% vaiable controlling the random number generator's seed, whether it should
% be fixed or not
rand_seed_bool=get(handles.rand_gen_seed_checkbox,'Value');

for idx_dye=1:min(length(handles.seq_params.dye),N_of_dyes) % goes through the different dye types
    % getting the dye parameters
    params.dye=handles.seq_params.dye{idx_dye};
    % is the "dye_prev" field exists, i. e. this function has been called
    % before
    if ~isempty(handles.seq_params.dye_prev)
        % for safety reasons
        if length(handles.seq_params.dye_prev)>=idx_dye
            params.dye_prev=handles.seq_params.dye_prev{idx_dye};
            % if dye parameters used for dye coordinate calculation have
            % been changed
            if params.dye.binding_angle~=params.dye_prev.binding_angle...
            || params.dye.binding_angle_dev~=params.dye_prev.binding_angle_dev...
            || params.dye.link_rad~=params.dye_prev.link_rad...
            || params.dye.N_of_labels_per_epitope~=params.dye_prev.N_of_labels_per_epitope...
            || params.dye.var_N_of_labels_per_epitope~=params.dye_prev.var_N_of_labels_per_epitope
                is_mol_coords_calc(:, idx_dye)={0};     % dye coordinates needs to be recalculated
            end
        end
    end
    
    for idx_struct=1:size(is_mol_coords_calc,1)  % goes through the individual patterns
        if rand_seed_bool
            rng(3*idx_struct+idx_dye);     % let this be the fix seed
        else
            % do nothing
        end
        if is_mol_coords_calc{idx_struct, idx_dye}==0     % if the positions and orientations on the pattern needs to be recalculated
            % getting the required inputs for "m_gen_mol_coord()"
            params.sample=handles.seq_params.sample{idx_struct};
            params.struct_coords=handles.seq_params.patterns.struct_coords{idx_struct};
            % if that structure has "label density" property
            if isfield(handles.seq_params.sample{idx_struct},'label_dens')
                params.sample.label_dens=handles.seq_params.sample{idx_struct}.label_dens(idx_dye,:);
            end
            % if that the structure is "fixed or random octagons"
            if isfield(handles.seq_params.sample{idx_struct},'oct_N_ofmol')
                params.sample.oct_N_ofmol=handles.seq_params.sample{idx_struct}.oct_N_ofmol(idx_dye,:);
            end
            % if that the structure is "fixed or random octagons"
            if isfield(handles.seq_params.sample{idx_struct},'oct_sig_N')
                params.sample.oct_sig_N=handles.seq_params.sample{idx_struct}.oct_sig_N(idx_dye,:);
            end
            % if that structure is "lattice"
            if isfield(handles.seq_params.sample{idx_struct},'lattice_mol')
                params.sample.lattice_mol=handles.seq_params.sample{idx_struct}.lattice_mol(idx_dye,:);
            end
            % if that the structure is "fixed vesicles"
            if isfield(handles.seq_params.sample{idx_struct},'ves_fix_N')
                params.sample.ves_fix_N=handles.seq_params.sample{idx_struct}.ves_fix_N(idx_dye,:);
            end
            % if that the structure is "random vesicles"
            if isfield(handles.seq_params.sample{idx_struct},'ves_rand_N_epitopes_mean')
                params.sample.ves_rand_N_epitopes_mean=handles.seq_params.sample{idx_struct}.ves_rand_N_epitopes_mean(idx_dye,:);
                if isfield(handles.seq_params.patterns.struct_coords{idx_struct},'vesicleDyeStaining_boolVect')
                    params.sample.vesicleDyeStaining_boolVect=handles.seq_params.patterns.struct_coords{idx_struct}.vesicleDyeStaining_boolVect(idx_dye,:);
                end
            end
            % if that the structure is "random vesicles"
            if isfield(handles.seq_params.sample{idx_struct},'ves_rand_epitopes_std')
                params.sample.ves_rand_epitopes_std=handles.seq_params.sample{idx_struct}.ves_rand_epitopes_std(idx_dye,:);
            end
            % if that the structure is "fixed rods"
            if isfield(handles.seq_params.sample{idx_struct},'rod_fix_N')
                params.sample.rod_fix_N=handles.seq_params.sample{idx_struct}.rod_fix_N(idx_dye,:);
            end
            % if that the structure is "random rods"
            if isfield(handles.seq_params.sample{idx_struct},'rod_N_ofmol')
                params.sample.rod_N_ofmol=handles.seq_params.sample{idx_struct}.rod_N_ofmol(idx_dye,:);
            end
            % if that the structure is "random rods"
            if isfield(handles.seq_params.sample{idx_struct},'rod_Sig_ofmol')
                params.sample.rod_Sig_ofmol=handles.seq_params.sample{idx_struct}.rod_Sig_ofmol(idx_dye,:);
            end
            
            % calculating the positions and the orientation for the actual pattern
            [act_labels]=m_gen_mol_coord(params);
            handles.seq_params.labels.init_cont_coords{idx_struct, idx_dye}=act_labels.init_cont_coords;
            handles.seq_params.labels.init_dye_coords{idx_struct, idx_dye}=act_labels.init_dye_coords;
            handles.seq_params.labels.init_orientation{idx_struct, idx_dye}=act_labels.init_orientation;
            handles.seq_params.labels.is_mol_coords_calc{idx_struct, idx_dye}=1;    % the positions and orientaton on the structure has benn recalculated
        end
    end
    handles.seq_params.dye_prev=handles.seq_params.dye; % dye parameters with which the calculation has been done
end

end

