function handles=m_gen_calc_nonspec_coords(handles)
% This function calculates the coordinates of nonspecific dye molecules.
% These dyes are placed in a cuboid centered at the origo. The sides of the
% cuboid are determined by the the Rayleigh length or the Gaussian PSF's
% peak value and the background level and the area captured by the CCD.

% PARENT: testSTORM.m
% CHILD: ~

aq=handles.seq_params.aq;
dye=handles.seq_params.dye;

qeff=aq.qeff;
preamp=aq.preamp;
actemgain=aq.actemgain;
colleff=0.28;
frame_fill_factor=aq.frame_fill_factor;
frame_rate=aq.frame_rate;
NA=aq.NA;
bg_level=aq.bg_level;
elpercount=aq.elpercount;
pixel_size=aq.pixel_size;

% initializing
handles.seq_params.labels.init_nonspec_coords{1,2}=[];
handles.seq_params.labels.init_nonspec_orientation{1,2}=[];

% the "x" and "y" range in which the dye molecules are distributed
range(1,1)=0;
range(1,2)=aq.pixel_size*aq.frame_size;
range(2,1)=0;
range(2,2)=aq.pixel_size*aq.frame_size;

switch get(handles.N_of_dyes_checkbox,'Value')
    case 0
        N_of_dyes=1;
    case 1
        N_of_dyes=2;
end

rand_seed_bool=get(handles.rand_gen_seed_checkbox,'Value');

if rand_seed_bool
    rng(1);     %let this be the fix seed
else
    %do nothing
end

z_depth_method='Rayleigh length';

switch z_depth_method
    case 'Rayleigh length'
        for idx_dye=1:min(length(handles.seq_params.dye),N_of_dyes)
            sigma_0=0.25*dye{idx_dye}.lambda/aq.NA;
            w_0=2*sigma_0;
            % Rayleight length
            z0=pi*w_0^2/dye{idx_dye}.lambda;
            % depth of "z" in which randomly distributed nonspecific dyes should be placed
            range(3,1)=-sqrt((49-1)*z0^2);
            range(3,2)=-range(3,1);

            x_length=range(1,2)-range(1,1);
            y_length=range(2,2)-range(2,1);
            z_length=range(3,2)-range(3,1);

            % volume in which randomly distributed background dyes should be placed
            volume=x_length*y_length*z_length;      % volume in nm^3
            N_mol=round(volume*dye{idx_dye}.nonspec_dens/1E9);   % total number of dye molecules
            
            if N_mol>0
                % handles.seq_params.labels.init_bg_coords{idx_dye}=(rand(3,N_mol)-0.5).*repmat([x_length;x_length;z_length],1,N_mol)+repmat([range(1,1);range(2,1);range(3,1)],1,N_mol);
                handles.seq_params.labels.init_nonspec_coords{1,idx_dye}=(rand(3,N_mol)-0.5).*repmat([x_length;x_length;z_length],1,N_mol);
                handles.seq_params.labels.init_nonspec_orientation{1,idx_dye}=[acos(1-rand(1,N_mol));2*pi*rand(1,N_mol)];
            elseif N_mol==0
                handles.seq_params.labels.init_nonspec_coords{1,idx_dye}=[];
                handles.seq_params.labels.init_nonspec_orientation{1,idx_dye}=[];
            end
        end
    case 'background level'
        for idx_dye=1:min(length(handles.seq_params.dye),N_of_dyes)
            sigma_0=0.25*dye{idx_dye}.lambda/NA;
            w_0=2*sigma_0;
            % Rayleight length
            z_R=pi*w_0^2/dye{idx_dye}.lambda;
            % ratio of PSF's peak amplitude of nonspecific labels with maximal spot
            % size and the background level
            nonpec_bg_ratio_min=0.5;
            % peak value of PSF photon density of nonspecific labels with maximal spot
            % size
            peak_w_max=nonpec_bg_ratio_min*bg_level*elpercount/(qeff*preamp*actemgain)/pixel_size^2;
            % maximal spot size of unspecific labels
            %w_max=7*w_0;
            w_max=sqrt(dye{idx_dye}.em_N_ofphotonpersec*colleff*frame_fill_factor/frame_rate/(peak_w_max*pi/2));
            % the photon density amplitude of PSF scales with ~1/w^2
            % depth of "z" in which randomly distributed nonspecific dyes should be placed
            %range(3,1)=-sqrt((w_max^2/w_0^2-1)*z_R^2);
            range(3,1)=-10*z_R;
            range(3,2)=-range(3,1);

            x_length=range(1,2)-range(1,1);
            y_length=range(2,2)-range(2,1);
            z_length=range(3,2)-range(3,1);

            % volume in which randomly distributed background dyes should be placed
            volume=x_length*y_length*z_length;      % volume in nm^3

            N_mol=round(volume*dye{idx_dye}.nonspec_dens/1E9);   % total number of dye molecules
            
            if N_mol>0
                % handles.seq_params.labels.init_bg_coords{idx_dye}=(rand(3,N_mol)-0.5).*repmat([x_length;x_length;z_length],1,N_mol)+repmat([range(1,1);range(2,1);range(3,1)],1,N_mol);
                handles.seq_params.labels.init_nonspec_coords{1,idx_dye}=(rand(3,N_mol)-0.5).*repmat([x_length;x_length;z_length],1,N_mol);
                handles.seq_params.labels.init_nonspec_orientation{1,idx_dye}=[acos(1-rand(1,N_mol));2*pi*rand(1,N_mol)];
            elseif N_mol==0
                handles.seq_params.labels.init_nonspec_coords{1,idx_dye}=[];
                handles.seq_params.labels.init_nonspec_orientation{1,idx_dye}=[];
            end
        end
end

end

