function [link_vect_mat, fi3_mat, theta3_mat, idx_mol]=m_gen_linker_rot(varargin)
% This function rotates the half sphere of the linker vectors so that the
% base of the half sphere to be perpendicular to the surface and then
% calculates the dye positions. It also mirrors the linker if needed and
% calculates the dye oriantations

% PARENT: m_gen_mol_coord_axons, m_gen_mol_coord_mesh, m_gen_mol_coord_tubules, m_gen_mol_coord_points
% CHILD: -

% VARIABLES
% params
% labels
% link_vec_sign
% R_y
% R_z
% N_mol_cont
% idx_mol
% contact_point
% idx_mol_cont
% idx_mol
% link_vect
% theta3
% fi3

switch length(varargin)
    case 7 % in case of non-line or non-ring patterns
        params=varargin{1};
        init_dye_coords=varargin{2};
        link_vec_sign=varargin{3};
        R_y=varargin{4};
        R_z=varargin{5};
        N_mol_cont=varargin{6};
        idx_mol=varargin{7};
        R_0=1;
    case 8  % in case of "segments", "lines" or "axons" patterns
        params=varargin{1};
        init_dye_coords=varargin{2};
        link_vec_sign=varargin{3};
        R_0=varargin{4};
        R_y=varargin{5};
        R_z=varargin{6};
        N_mol_cont=varargin{7};
        idx_mol=varargin{8};
    otherwise
        error('Error in the number of input parameters of "m_gen_linker_rot.m".')
end

% calculating dye positions
link_vect_mat=zeros(3, N_mol_cont);
theta3_mat=zeros(1, N_mol_cont);
fi3_mat=zeros(1, N_mol_cont);
for idx_mol_cont=1:N_mol_cont % goes through the dye molecules on the contact point
    link_vect=init_dye_coords(:,idx_mol_cont);
    link_vect=R_z*R_y*R_0*link_vect*link_vec_sign(idx_mol_cont);    % vector of the dye's linker
    theta3=acos(link_vect(3)/params.dye.link_rad); % polar angle of the linker
    fi3=atan2(link_vect(2),link_vect(1)); % azimuth angle of the linker
    if isnan(fi3)   % if the vedtor is parallel with the "z" axis...
        fi3=0;      % let the azimuth agnle be zero
    end
    link_vect_mat(:, idx_mol_cont)=link_vect;
    theta3_mat(:, idx_mol_cont)=theta3;
    fi3_mat(:, idx_mol_cont)=fi3;
    idx_mol=idx_mol+1;
end

end

