function [ labels ] = m_gen_mol_coord_points( point_coords, params )
% This function places contact point on patterns consisting of points
% (lattice and octagon pattern). First calculates the number of contact
% point on the individual points, then connects the dye molecules to the
% contact points. The dye molecules are placed one a full sphere with
% uniform distribution.

% PARENT: m_gen_mol_coord.m
% CHILDS: m_gen_linker.m, m_gen_linker_rot.m

N_point=size(point_coords,2);   % total number of vertices or lattice points
switch params.sample.name
    case 'lattice'
        N_mean=params.sample.lattice_mol; % mean number of contact points on each lattice point
        N_var=0;        % variance of the number of dye molecules on each lattice point
    case 'octagons'
        N_mean=params.sample.oct_N_ofmol;   % mean number of contact points on each vertex
        N_var=params.sample.oct_sig_N;  % variance of the number of dye molecules on each vertex
end
P=1-N_var/N_mean;   % success probability in each trial

% if the probability has negative or zero value...
if P<=0
    P=0;        % count it as zero
    N_trials=0;
    disp('Too large variance was given to the octagons...')
% if "N_mean" is zero
elseif isnan(P)
    P=0;
    N_trials=0;
% if P has meaningful value
else
    N_trials=round(N_mean/P);    % number of trials in binomial distribution
end

N_cont_point=m_gen_random('binomial', N_trials,P,1,N_point);      % number of contact points on the individual vertices
%N_cont_point=binornd(N_trials,P,1,N_point);      % number of contact points on the individual vertices
N_cont=sum(N_cont_point);                % total number of contact points

% getting the linker vectors
[labels, N_mol_cont, link_vec_sign]=m_gen_linker(N_cont, params);

% indexing, helps determining which dye molecule to which octagon belongs
point_indexes=zeros(1,N_cont);
idx_low=1;
idx_up=0;
for idx_point=1:length(N_cont_point)
    idx_up=idx_up+N_cont_point(idx_point);
    point_indexes(idx_low:idx_up)=idx_point;
    idx_low=idx_up+1;
end


idx_mol=1;      % index of the actual dye molecule
for idx_cont=1:N_cont
    idx_point=point_indexes(idx_cont); % index of the actual vertex
    
    % the linker vectors should not be rotated
    R_y=[1,0,0;...
        0,1,0;...
        0,0,1];
    R_z=[1,0,0;...
        0,1,0;...
        0,0,1];
    
    contact_point=point_coords(:, idx_point);       % actual contact point coordinates

    % extending the linker vectors to a full sphere from a half sphere
    [link_vect, fi3, theta3, idx_mol_next]=m_gen_linker_rot(params, labels.init_dye_coords(:,idx_mol:idx_mol+N_mol_cont(idx_cont)-1), link_vec_sign, R_y, R_z,  N_mol_cont(idx_cont), idx_mol);
    
    for idx_mol_cont=1:N_mol_cont(idx_cont)
        labels.init_dye_coords(:,idx_mol+idx_mol_cont-1)=contact_point+link_vect(:, idx_mol_cont);   % variable containing dye coordinates in 3xM_mol array
        labels.init_orientation(:,idx_mol+idx_mol_cont-1)=[fi3(idx_mol_cont);theta3(idx_mol_cont)];   % variable containing dye orientations (azimuth, polar angles) in 3xM_mol array
    end
    labels.init_cont_coords(:,idx_cont)=contact_point;  % variable containing contact point coordinates in 3XN_cont array
    idx_mol=idx_mol_next;
    
end

end

