function output=m_gen_random(name,varargin)

if ~ischar(name)
    error('Invalid input.')
end

if isempty(varargin)
    error('Missing Parameters')
end

flagDistBuiltIn=0;

switch name
    case 'exponential'
        output = m_gen_exponential(varargin{:});
    case 'lognormal'
        output = m_gen_lognormal(varargin{:});
    case 'normal'
        output = m_gen_normal(varargin{:});
    case 'poisson'
        if flagDistBuiltIn==1
            if length(varargin)~=1
               error('Too much input for poisson distribution.')
            end
            lambda=varargin{1};
            output = poissrnd(lambda);
        elseif flagDistBuiltIn==0
            output = m_gen_poisson(varargin{:});
        end
    case 'uniform'
        output = m_gen_uniform(varargin{:});
    case 'binomial'
        if flagDistBuiltIn==1
        	N=varargin{1};
            P=varargin{2};
            dim1=varargin{3};
            dim2=varargin{4};
            output = binornd(N, P, dim1, dim2);
        elseif flagDistBuiltIn==0
            output = m_gen_binomial(varargin{:});
        end
    case 'reset'
        rng('default');
    otherwise
        error(['Unknown distribution or action: ',num2str(name)]);
end
end

function output=m_gen_exponential(varargin)
if length(varargin)~=1
    error('To much input for exponential distribution.')
end
mu=varargin{1};
output_size=size(mu);
output = -mu .* log(rand(output_size));
end

function output=m_gen_lognormal(varargin)
if length(varargin)~=2
    error('Two inputs are needed for lognormal distribution.')
end
mu=varargin{1};
sigma=varargin{2};
output_size=size(mu);

if any(sigma(:) < 0)
    error('Negative sigma input.')
end
output = exp(randn(output_size) .* sigma + mu);
end

function output=m_gen_normal(varargin)
mu=varargin{1};
sigma=varargin{2};
output_size=size(mu);

if any(sigma(:) < 0)
    error('Negative sigma input.')
end
output = randn(output_size) .* sigma + mu;
end

function output=m_gen_poisson(varargin)
if length(varargin)~=1
    error('Too much input for poisson distribution.')
end
lambda_lim=100;
max_k = lambda_lim+sqrt(lambda_lim)*6;

lambda=varargin{1};
output=zeros(size(lambda));
for idx=1:length(lambda(:))
    if lambda(idx)<lambda_lim
        k=0;
        p = rand(1);
        P = exp(-lambda(idx));
        sum=P;
        if (sum>=p)
            output(idx)=0;
        else
            for k=1:max_k
                P=P*lambda(idx)/k;
                sum=sum+P;
                if (sum>=p)
                    break
                end
            end
            output(idx)=k;
        end
    else
        output(idx)=round(randn(1)*sqrt(lambda(idx))+lambda(idx));
    end
end

end


function output=m_gen_uniform(varargin)
if length(varargin)~=2
    error('Two inputs are needed for uniform distribution.')
end
mu=(varargin{1}+varargin{2})/2;
sigma=(varargin{2}-varargin{1})/2;
output_size=size(mu);
if any(sigma(:) < 0)
    error('Negative sigma input.')
end
output = mu + sigma .* (2*rand(output_size)-1);
end


function output=m_gen_binomial(varargin)
    if length(varargin)~=4
        error('Four inputs are needed for binomial distribution.')
    end
    N=varargin{1};      % number of trials
    P=varargin{2};      % probability of success
    dim1=varargin{3};   % first dimension of the output matrix
    dim2=varargin{4};   % second dimension of the output matrix
    if P>1
        error('The probability for binomial distribution is larger than 1.');
    end
    trials=rand(N, dim1, dim2);
    trials(trials>P)=1;
    trials(trials<=P)=0;
    binomDist=N-sum(trials,1);

    if dim1~=1
            output(:,:)=binomDist(1,:,:);
    else    % FUCK MATLAB AGAIN, why the hell an "1xMxN" ("b") becomes an NxM array in "a(:,:)=b(1,:,:)" in the substitution but only when "M" is "1"????????????
            output(:,:)=permute(binomDist(1,:,:), [1,3,2]);
    end
        
end