function [ vert_coords ] = m_gen_read_snake( params )
% This function reads the content of a "snake" file created by the JFilament
% ImageJ plugin. This file format can contain multiple segmented lines.

% PARENT: m_gen_struct.m
% CHILDS: -

% VARIABLES

% filename: the name of the file to open
% scaling: used to increase the size of the pattern
% displacement: desired shift to the pattern
% line_act: content of the actual line in the file, string
% idx_curve: index of the actual curve
% idx_curve: index of the actual curve
% idx_vert: index of the actual vertice
% line_content: content of the actual line in the file, array
% vert_coords: cell containing the arrays containing coordinates of the verices on the individual segmented lines


filename=params.sample.seg_filename;        % the name of the file to open
scaling=params.sample.seg_scaling;          % used to increase the size of the pattern
displacement=[params.sample.seg_disp_x;params.sample.seg_disp_y;params.sample.seg_disp_z];  % desired shift to the pattern
theta=params.sample.seg_rot_theta*pi/180;	% polar angle of rotation
fi=params.sample.seg_rot_fi*pi/180;         % azimuth angle of rotation
% rotation matrices
R_y=[cos(theta),0,sin(theta);...
	0,1,0;...
    -sin(theta),0,cos(theta)];
R_z=[cos(fi),-sin(fi),0;...
    sin(fi),cos(fi),0;...
    0,0,1];

% if some of the displacement's values are empty
if size(displacement,1)~=3
   error('Wrong displacement for segments'); 
end

fileID=fopen(filename);     % opens the given file
line_act=fgetl(fileID);     % reads the first line of the file

idx_curve=0;        % index of the actual curve
vert_coords={};     % cell containing the arrays containing coordinates of the verices on the individual segmented lines
while ischar(line_act)      % while it reaches the end of file
   if line_act=='#'         % '#' signs the beginning of the data of a curve
       line_act=fgetl(fileID);  % this line does not contain any information
       idx_curve=idx_curve+1;   % index of the actual curve
       idx_vert=0;              % index of the actual vertice
       line_act=fgetl(fileID);  % reading the first line of the data of the actual curve containing the information
       while and(line_act~='#',ischar(line_act))    % while it reaches the next curve
           idx_vert=idx_vert+1;   % index of the actual vertex of the curve
           line_content=str2double(strsplit(line_act,'\t'));    % split the string into list
           line_content(4)=-line_content(4);            % inverting "y" coordinates, converting ImageJ's cordinate system to Matlab's
           vert_coords{idx_curve}(1:3,idx_vert)=R_z*R_y*((line_content(3:5)*scaling)'+displacement);    % the "x" coordinate
           line_act=fgetl(fileID);      % get the content of the next line
       end
   else
       line_act=fgetl(fileID);  % reads the lines until reaches the first curve 
   end
end
fclose(fileID);     % close the file
end

