function [ vert_coords ] = m_gen_struct_disks( params )
% This functions calculates the starting and ending point of three sarcomeres
% similarily to in case of lines. Coordinates belonging to the individual
% sarcomeres are placed in different cell elements. It goues through the sarcomeres
% abd calculats "x", "y" and "z" coordinates of their starting and ending
% points. These coordinates are placed in 3x2 array.

% PARENT: m_gen_struct.m
% CHILDS: -

% VARIABLES

% fi: polar angles between sarcomeres ang the "z" axis
% theta: azimuth angle of sarcomeres
% z_0: "z" position sarcomeres' centers
% length_sarcomere: size of lines pattern in nm
% idx_sarcomere: index of the actual sarcomere
% x: variables containing the "x" coordinates if the sarcomere's starting and ending points
% y: variables containing the "y" coordinates if the sarcomere's starting and ending points
% z: variables containing the "z" coordinates if the sarcomere's starting and ending points
% vert_point: variable containing the actual sarcomere's starting and ending coordinates in 3x2 array
% vert_coords: cell containing coordinates of starting and ending points

theta=params.sample.sarcomere_theta*pi/180;    % polar angles between sarcomeres ang the "z" axis
fi=params.sample.sarcomere_fi*pi/180; % azimuth angle of sarcomeres
x_0=params.sample.sarcomere_x; % "x" position sarcomere' centers
y_0=params.sample.sarcomere_y; % "y" position sarcomere' centers
z_0=params.sample.sarcomere_z; % "z" position sarcomere' centers
length_sarcomere=params.sample.sarcomere_length; % size of lines pattern in nm

N_sarcomere=size(length_sarcomere,2);

vert_coords=cell(1,N_sarcomere);  % coordinates of starting and ending points

% calculate molecule positions
for idx_sarcomere=1:N_sarcomere
    % memory allocation and initialization
    vert_coords{idx_sarcomere}=zeros(3,2);
    x=x_0(idx_sarcomere)+[-sin(theta(idx_sarcomere))*cos(fi(idx_sarcomere)),sin(theta(idx_sarcomere))*cos(fi(idx_sarcomere))]*length_sarcomere(idx_sarcomere)/2;
    y=y_0(idx_sarcomere)+[-sin(theta(idx_sarcomere))*sin(fi(idx_sarcomere)),sin(theta(idx_sarcomere))*sin(fi(idx_sarcomere))]*length_sarcomere(idx_sarcomere)/2;
    z=z_0(idx_sarcomere)+[-cos(theta(idx_sarcomere)),cos(theta(idx_sarcomere))]*length_sarcomere(idx_sarcomere)/2;
    vert_point=[x;y;z]; % coordinates of starting and ending points of the actual sarcomere
    vert_coords{idx_sarcomere}=vert_point;   % cell array containing the coordinates
      
end

end

