function [ point_coords ] = m_gen_struct_lattice( params )
% This function calculates the positions of the contact points in the
% lattice. Places the lattice points equidistantly in the "x-y" plane with
% the given grid distance, then calculates the "z" positions with the given
% tilting angle. in case of tilting the lattice is rotated around the "y"
% axis. The center of the latttice coincides with the center of the
% coordinate sysrem.

% PARENT: m_gen_struct.m
% CHILDS: -

% VARIABLES

% dim: size of lattice in nm
% dist_grid: distance between the points of the lattice in nm in the "x-y" plane
% alpha: rotation angle around "y axis"  in rad
% N_lat: number of lattice points in "x" and "y" dimension
% point_coords: 3xN_lat array containing the coordinates of the lattice points

dim=params.sample.lattice_dim; % size of lattice in nm
z_pos=params.sample.lattice_z_pos; % axial position of the lattice
dist_grid=params.sample.lattice_dist_grid; % distance between the points of the lattice in nm
alpha=params.sample.lattice_alpha*pi/180; % angle in z dimension in rad
N_lat=floor(dim/dist_grid)+1; % N of points in one dimension 


point_coords=[(repmat((0:N_lat-1),1,N_lat))*dist_grid-0.5*dim;... % x coord
    (reshape(repmat((0:N_lat-1),N_lat,1),[],1)')*dist_grid-0.5*dim;... % y coord
    (repmat((0:N_lat-1),1,N_lat)-((N_lat-1)/2)).*(tan(alpha)*dist_grid)+z_pos]; % z coord

end

