function [ cont_coords ] = m_gen_struct_octagons( params )
% This function calculates the positions of the vertice of the octagons.
% It goes through the vertices and octagons in order, applies the
% appropriate rotation to every vertice and shifts them to the center of
% the actual octagon.

% PARENT: m_gen_struct.m
% CHILDS: -

% VARIABLES
% point_coords:
% N_oct: number of octagons
% N_vert: total number of vertices
% idx_oct_prev: index of the octagon on which the previously calculated vertices located
% idx_oct: index of the actual octagon
% idx_vertex_local: local index of the actual vertice on the actual octagon
% pos_oct: position of the center of the actual octagon
% rad_oct: radius of the actual octagon
% rand_for_alpha: random number for the rotation around the axis of the octagon
% aplha: angle of rotation around the axis of the octagon
% idx_vertex: index of the vertex (contact point) on the octagon
% contact_point: coordinates of the actual contact point (vertex)
% cont_coords: coordinates of the vertices in an 3xN_vert array


% if fixed coordinates were chosen
if strcmp(params.sample.oct_sim_type,'fix')
    point_coords=[...
        params.sample.oct_fix_x;...
        params.sample.oct_fix_y;...
        params.sample.oct_fix_z];
% if random coordinates were chosen
elseif strcmp(params.sample.oct_sim_type,'random')
    oct_N_ofoct=params.sample.oct_N_ofoct;  % number of vesicules
    oct_dim=params.sample.oct_dim;      % "x" and "y" extension of area in which the vesicules are distributed
    oct_zmax=params.sample.oct_zmax;    % "z" extension of area in which the vesicules are distributed
    point_coords=[(rand(1,oct_N_ofoct)-0.5)*oct_dim;...
        (rand(1,oct_N_ofoct)-0.5)*oct_dim;...
        (rand(1,oct_N_ofoct)-0.5)*oct_zmax];
else
    error('Error in vesicles coordinates');
end


N_oct=size(point_coords,2);   % number of octagons
N_vert=N_oct*8;         % total number of vertices
cont_coords=zeros(3,N_vert);

idx_oct_prev=0;
for idx_vertex=1:N_vert     % goes trhough the vertices
    idx_oct=floor((idx_vertex-1)/8)+1;  % index of the actual octagon
    idx_vertex_local=mod(idx_vertex,8);     % index of the vertice on the octagon, 1-8
    pos_oct=point_coords(:, idx_oct); % get center positions
    rad_oct=params.sample.oct_rad;    % get radius
    % if the vertex is on a new octagon...
    if idx_oct~=idx_oct_prev
        rand_for_alpha=0.125*rand(1);   % generate new random number
    end
    alpha=2*pi*(rand_for_alpha+idx_vertex_local*0.125);  % angle for the contact points
    % coordinates of the veritces (contact_points)
    cont_coords(:,idx_vertex)=pos_oct+rad_oct*[...
        cos(alpha);...
        sin(alpha);...
        0];
    idx_oct_prev=idx_oct;
end

end
