function [ rod_coords ] = m_gen_struct_rods( params )
% This function generates the center positions of rods. In case of
% user given coordinates it simply places the given coordinates in an
% array. Otherwise randomly places the rod centers in the given area.

% PARENT: m_gen_struct.m
% CHILDS: -

% if fixed coordinates were chosen
if strcmp(params.sample.rod_sim_type,'fix')
    rod_coords.pos=[...
        params.sample.rod_fix_x;...
        params.sample.rod_fix_y;...
        params.sample.rod_fix_z];
    
    rod_N=size(params.sample.rod_fix_x,2);  % number of rods
    rod_coords.theta=zeros(rod_N,1);
    rod_coords.fi=params.sample.rod_fix_phi;
    
% if random coordinates were chosen
elseif strcmp(params.sample.rod_sim_type,'random')
    rod_dim=params.sample.rod_dim;      % "x" and "y" extension of area in which the rods are distributed
    rod_zmax=params.sample.rod_zmax;    % "z" extension of area in which the rods are distributed
    rod_N=params.sample.rod_N_ofrod;  % number of rods
    rod_coords.pos=[...
        (rand(1,rod_N)-0.5)*rod_dim;...
        (rand(1,rod_N)-0.5)*rod_dim;...
        (rand(1,rod_N)-0.5)*rod_zmax];  % 3x(ves_fix_N/ves_N_ofves) array containing coordinates of rods
    
    % generate spherical angles for rod rotation, only rotated in the "x-y"
    % plane uniformly
    rod_coords.theta=zeros(rod_N,1);
    rod_coords.fi=2*pi*rand(rod_N,1);
    rod_coords.fi(isnan(rod_coords.fi))=0;      % if the vector is parallel with the "z" axis, let the azimuth agnle be zero
else
    error('Error in rod coordinates');
end

end

