function [ mesh_coords]=m_gen_struct_star(params)
% The star pattern conist of contact points randomly distributed in 16 
% isosceles triangles in th "x-y" plane. The dye molecules are linked to
% these contact points. The triagles has a vertex angle of 15� (2*"alpha")
% and height of "params.star_dim". All triangles contacts each other at the
% appex. The adjacent legs of the neighbouring triangles form an angle of
% 7.5�. The triangles are rotated with the right angle. Reapeting this
% 16 times forms the star pattern.


% PARENT: m_gen_struct.m
% CHILDS: -

% VARIABLES
% a=params.sample.star_dim;   % height of isosceles triangles in nm
% alpha: half the vertex angle in rad 
% b: half the base of isosceles triangles in nm
% z: z position of the veritces
% N_arm: number of arms of the star pattern
% temp: temporal structure that hellp arrange the information into more convenient format
% idx_arm: index of the actual arm
% vert_coords: cell containing 3x3 array containing the coordinates of vertices of the triangle
% N_facet: number of triangles
% indices: indices of the triangle's vertices in the "temp.vert_coords" array
% mesh_coords.vert_coords: array containing the coordinates of the individual vertices
% mesh_coords.facet_verts: array containing information about which vertice to which facet belongs

N_arm=16;   % number of arms of the star pattern

a=params.sample.star_dim;   % height of isosceles triangles in nm
alpha=2*pi/N_arm/3;   % half the vertex angle in rad 
b=a*tan(alpha);     % half the base of isosceles triangles in nm

z=params.sample.star_z; % z position of the veritces

mesh_coords=[];             % structure containing tha face normals and the vertex coordinates
temp=[];

vert_coords=cell(N_arm,1);

% calculationg the coordinates of the vesicles on the arms
for idx_arm=1:N_arm
    vert_coords{idx_arm}=[0,a,a;0,b,-b;z,z,z];
    % rotating the arm in the appropriate position        
    vert_coords{idx_arm}=[...
        cos((idx_arm-1)*3*alpha),-sin((idx_arm-1)*3*alpha),0;...
        sin((idx_arm-1)*3*alpha),cos((idx_arm-1)*3*alpha),0;...
        0,0,1]...
        *vert_coords{idx_arm};
    vert_coords{idx_arm}=vert_coords{idx_arm}';
end

% arranging the infomation into more convenient format

N_facet=size(vert_coords,1);        % number of triangles

% temporal variables containing the arranged informations
% needs because "unique()" and "ismember()" does not have 'columns' property
temp.vert_coords=unique(cat(1,vert_coords{:,1}),'rows');
temp.facet_verts=zeros(N_facet,3);

% finding which vertice to which facet belongs
for idx_facet=1:N_facet
    for idx_vert=1:3
        indices=ismember(temp.vert_coords,vert_coords{idx_facet,1}(idx_vert,1:3),'rows');
        temp.facet_verts(idx_vert,idx_facet)=find(indices);
    end
end

% output variables containing the information arranged in convenient format
mesh_coords.vert_coords=temp.vert_coords';
mesh_coords.facet_verts=temp.facet_verts;

end
