function [ vesicle_coords ] = m_gen_struct_vesicles( params )
% This function generates the center positions of vesicles. In case of
% user given coordinates it simply places the given coordinates in an
% array. Otherwise randomly places the vesicle centers in the given area.
% The overlapping of randomly placed vesicles are prevented, in case of of
% ellipsoidal vesicles the larget axis is used to achieve this.

% PARENT: m_gen_struct.m
% CHILDS: -

% if fixed coordinates were chosen
if strcmp(params.sample.ves_sim_type,'fix')
    vesicle_coords.pos=[...
        params.sample.ves_fix_x;...
        params.sample.ves_fix_y;...
        params.sample.ves_fix_z];
    
    R_ves=params.sample.ves_fix_rad;    % radius of the individual vesicules
    vesicle_coords.rad=R_ves;
    
    ves_N=size(params.sample.ves_fix_x,2);  % number of vesicles
    vesicle_coords.ves_N=ves_N;
    
    % rotation angles of the ellipsoid
    %theta_rot_ellipsoids=acos(1-2*rand(N_ves,1));     % angle for the ellipsoid's rotation around the "y" axis
    vesicle_coords.theta=zeros(ves_N,1);      % angle for the ellipsoid's rotation around the "y" axis
    vesicle_coords.fi=2*pi*rand(ves_N,1);  % angle for the ellipsoid's rotation around the "z" axis
    %fi_rot_ellipsoids=zeros(N_ves,1);  % angle for the ellipsoid's rotation around the "z" axis
    
    
% if random coordinates were chosen
elseif strcmp(params.sample.ves_sim_type,'random')
    %% vesicles on synaptosomes
    % these parameters must be set by hand:
    synaptosome_posX=params.sample.ves_rand_synaptosomes_pos(1,:);      % "x" psoition of synaptosomes' centres
    synaptosome_posY=params.sample.ves_rand_synaptosomes_pos(2,:);      % "y" psoition of synaptosomes' centres
    synaptosome_posZ=params.sample.ves_rand_synaptosomes_pos(3,:);      % "z" psoition of synaptosomes' centres
    synaptosome_rad=params.sample.ves_rand_synaptosomes_rad;        % radius of each synaptosome
    synaptosome_vesN=params.sample.ves_rand_synaptosomes_N;    % number of vesicles in each synaptosome

    synaptosome_pos=[synaptosome_posX; synaptosome_posY; synaptosome_posZ]; % 3x'synaptosome number' array containing the synaptosome positions
    synaptosome_N=size(synaptosome_pos,2);      % number of synaptosomes
    
    ves_N_synaptosomes=sum(synaptosome_vesN);       % total number of vesicles in every synaptosome
    
    % initializing the structure array containing the synaptosome 
    % parameters
    synaptosome=repmat(struct('pos', [],'rad', [],'ves_N', [],'ves_pos', [],'ves_rad', []), [1,synaptosome_N]);
    
    % parameters describing the distribution of the vesicle radius
    R_mean_max=max(params.sample.ves_rand_rad_mean);   % mean radius of vesicles, largest axis in case of ellipsoidal vesicles
    R_mean=params.sample.ves_rand_rad_mean;   % mean radii of vesicles' axes
    R_sig=params.sample.ves_rand_rad_std;    % sigma of radius of vesicles
    
    

    
    % choose what kind is staining should be used:
    stainingType='all';     % staing every vesicle with eachdye
    %stainingType='manual';   % set maunally what portion of vesicles should be stained with which dye
    
    % for Pedro
    %stainingType='percentages'; % VAMP-2 and A-syn with the given 90% and 30% percentages, mCLING with 50% that contain A-syn and VAMP-2
    % dye1: Atto647N, mCLING
    % dye2: AF 568, A-syn
    % dye3: AF 488, VAMP-2
    
    % structure array containing which vesicles should be stained with
    % which dye in each synaptosome
    vesicleDyeStaining_boolStruct=repmat(struct('val', []), 1, synaptosome_N);
    % loop over the synaptosomes
    for idxSynap=1:synaptosome_N
        % parameters of the actual synaptosome
        synaptosome(idxSynap).pos=synaptosome_pos(:, idxSynap);     % position of the actual synaptosome
        synaptosome(idxSynap).rad=synaptosome_rad(1, idxSynap);     % radius of the actual synaptosome
        synaptosome(idxSynap).vesN=synaptosome_vesN(1, idxSynap);   % number of vesicles in the actual synaptosome
        synaptosome(idxSynap).ves_pos=[];       % position of the vesicles in the actual synaptosome
        synaptosome(idxSynap).ves_rad_max=[];       % largest axis of the vesicles in the actual synaptosome
        synaptosome(idxSynap).ves_rad=[];       % radii of the ellipsoidal vesicles in the actual synaptosome
        
        
        switch stainingType
            case 'all'
                border=0;      % border area of the slices, in nm
                % stain all vesicles
                vesicleDyeStaining_boolVect_temp=true(3, synaptosome(idxSynap).vesN);
                % no spefific rule for the vesicle separation in this case:
                stainedVesicles_phiMinVect=zeros(1, synaptosome(idxSynap).vesN);
                stainedVesicles_phiMaxVect=2*pi*ones(1, synaptosome(idxSynap).vesN);
            case 'manual'
                border=50;      % border area of the slices, in nm
                    vesicleDyeStaining_boolVect_temp=false(3, synaptosome(idxSynap).vesN);
                    stainedVesicles_phiMinVect=zeros(1, synaptosome(idxSynap).vesN); % variable defining the vesicle angular distribution inside the synaptosomes
                    stainedVesicles_phiMaxVect=zeros(1, synaptosome(idxSynap).vesN); % variable defining the vesicle angular distribution inside the synaptosomes
                
                for idx_dye=1:3
                    % set which part of the vesicles should be stained
                    % with which dye and in which slice of the synaptosome
                    % should the they be placed
                    switch idx_dye
                        case 1
                            starting_norm_idx=0;
                            ending_norm_idx=0.33;
                            phi_min=0;
                            phi_max=2*pi/3;
                        case 2
                            starting_norm_idx=0.33;
                            ending_norm_idx=0.66;		% the anding value should be slightly larger than 1 if all vesicles are to be stained 
                            phi_min=2*pi/3;
                            phi_max=2*pi*2/3;
                        case 3
                            starting_norm_idx=0.66;
                            ending_norm_idx=1.1;
                            phi_min=2*pi*2/3;
                            phi_max=2*pi;
                    end
                    vesicleDyeStaining_boolVect_actDye_temp=false(1, synaptosome(idxSynap).vesN);
                    
                    % indices of vesicles belonging to this staining,
                    % arbitrary rounding rule
                    starting_idx=uint16(max(1,ceil(starting_norm_idx*synaptosome(idxSynap).vesN)));
                    ending_idx=uint16(min(synaptosome(idxSynap).vesN,ceil(ending_norm_idx*synaptosome(idxSynap).vesN)-1));
                    
                    vesicleDyeStaining_boolVect_actDye_temp(starting_idx:ending_idx)=true;
                    
                    % angular distribution af the vesicles, which slice
                    % should the be placed in, in this synaptosome
                    stainedVesicles_phiMinVect(starting_idx:ending_idx)=phi_min;
                    stainedVesicles_phiMaxVect(starting_idx:ending_idx)=phi_max;
                    
                    % boolean vector which containing which vesicles should
                    % be stained with which dye in this synaptosome
                    vesicleDyeStaining_boolVect_temp(idx_dye, :)=vesicleDyeStaining_boolVect_actDye_temp;
                end
            case 'percentages'
                border=0;      % border area of the slices, in nm
                % this boolean vector should be changed later
                vesicleDyeStaining_boolVect_temp=false(3, synaptosome(idxSynap).vesN);
                % no spefific rule for the vesicle separation in this case:
                stainedVesicles_phiMinVect=zeros(1, synaptosome(idxSynap).vesN);
                stainedVesicles_phiMaxVect=2*pi*ones(1, synaptosome(idxSynap).vesN);
        end
        
        
        % loop throught the vesicles in the synaptosome
        vesIsPlaced_boolVect=false(1, synaptosome(idxSynap).vesN);
        N_placedVes=0;      % index of the actual vesicle
        actVesTries_N=0;      % number of tries given in placing the actual vesicle (the vesicles can not overlap)
        synaptosome_rand_phi=2*pi*rand(1); % randomly rotate the slices in the synaptosome around the "z" axis
        for idx_ves=1:synaptosome(idxSynap).vesN
            while true      % I like to live dangerously...
                
                actVesTries_N=actVesTries_N+1;  % one more try was given in placing the actual vesicle
                
                % angles defining the slice that the actual vesicle belongs
                % to
                phi_min=stainedVesicles_phiMinVect(idx_ves)+synaptosome_rand_phi;
                phi_max=stainedVesicles_phiMaxVect(idx_ves)+synaptosome_rand_phi;
                actVesPos_phi=rand(1,1)*(phi_max-phi_min)+phi_min;
                actVesPos_theta=acos((rand(1,1)-0.5)*2);
                actVesPos_r=synaptosome(idxSynap).rad*sqrt(rand(1,1));
                actVesPos=synaptosome(idxSynap).pos+[...
                    actVesPos_r*sin(actVesPos_theta)*cos(actVesPos_phi);...
                    actVesPos_r*sin(actVesPos_theta)*sin(actVesPos_phi);...
                    actVesPos_r*cos(actVesPos_theta)];
                
                % calculating the minimum vesicle position distance from
                % the synaptosome center that is required for the vesicle
                % not to fall inside the border area
                if actVesPos_phi<=(phi_min+phi_max)/2
                    if border==0 % to avoid 0/0
                        r_min=0;
                    else
                        r_min=border/sin(actVesPos_phi-phi_min);
                    end
                elseif actVesPos_phi>=(phi_min+phi_max)/2
                    if border==0 % to avoid 0/0
                        r_min=0;
                    else
                        r_min=border/sin(phi_max-actVesPos_phi);
                    end
                else
                    error('some problem with the vesicle spherical coordinates.')
                end
                
                %             % place the vesicle randomly in a cube determined by the
                %             % synaptosome radius, will discard it later, if it is outside
                %             % of the sphere
                %             actVesPos=(rand(3,1)-0.5)*2*synaptosome(idxSynap).rad+synaptosome(idxSynap).pos;
                
                % generate the radius for the actual vesicle
                actVesRad_max=R_mean_max+R_sig.*randn(1, 1);
                actVesRad=R_mean+R_sig.*randn(3, 1);
                
                % examine whether the actual vesicle is inside the actual
                % synaptosome and is not located inside the border area of
                % the slice
                %synaptosome_withinBool=sum((actVesPos-synaptosome(idxSynap).pos).^2)<=(synaptosome(idxSynap).rad-actVesRad_max)^2;
                synaptosome_withinBool=actVesPos_r<=(synaptosome(idxSynap).rad-actVesRad_max) && actVesPos_r*sin(actVesPos_theta)>r_min;
                
                % examine whether the actual vesicle overlaps with any of the
                % previously placed vesicles in the synaptosome
                if ~isempty(synaptosome(idxSynap).ves_pos)      % if it is not the first vesicle in the synaptosome
                    vesDistances=sum((synaptosome(idxSynap).ves_pos-repmat(actVesPos, [1, size(synaptosome(idxSynap).ves_pos,2)])).^2,1);   % square of distances of the actual and the previously placed vesicles
                    vesRadSums=synaptosome(idxSynap).ves_rad_max+repmat(actVesRad_max, [1, size(synaptosome(idxSynap).ves_rad_max,2)]);         % sum of radii of the actual and the previously placed vesicles
                    vesDistances_RadSums_Bool=vesDistances>=vesRadSums.^2;      % examine whether actual vesicle overlaps with any other in the synaptosome
                    vesicle_distBool=prod(vesDistances_RadSums_Bool);       % true if the actual vesicle does not overlap with any other in the synaptosome
                else        % if it is the first vesicle in the synaptosome
                    vesicle_distBool=true;
                end
                
                % if the actual vesicle is inside the synaptosome and does not
                % overlap with any of the previously placed vesicles, add it to
                % the parameters of the actual synaptosome and leap to the next
                % vesicle
                if synaptosome_withinBool && vesicle_distBool
                    N_placedVes=N_placedVes+1;
                    actVesTries_N=0;
                    synaptosome(idxSynap).ves_pos=[synaptosome(idxSynap).ves_pos, actVesPos];
                    synaptosome(idxSynap).ves_rad_max=[synaptosome(idxSynap).ves_rad_max, actVesRad_max];
                    synaptosome(idxSynap).ves_rad=[synaptosome(idxSynap).ves_rad, actVesRad];
                    vesIsPlaced_boolVect(idx_ves)=true;
                    break;
                end
                
                % if too many tries was given in placing the vesicle, consider
                % not any more vesicle can be placed in this synatosome, leap
                % to the next synaptosome
                if actVesTries_N>100
                    break;
                end
                
            end
        end
        
        disp([num2str(N_placedVes), '/', num2str(synaptosome(idxSynap).vesN),' vesicles were placed in synaptosome numer ', num2str(idxSynap)])
        
        switch stainingType
            case 'all'
                % all vesicles are labelled with each kind of dyes
                vesicleDyeStaining_boolStruct(idxSynap).val=vesicleDyeStaining_boolVect_temp(:, vesIsPlaced_boolVect);
            case 'manual'
                vesicleDyeStaining_boolStruct(idxSynap).val=vesicleDyeStaining_boolVect_temp(:, vesIsPlaced_boolVect);
            case 'percentages'
                vesicleDyeStaining_boolVect_temp=false(3, synaptosome(idxSynap).vesN);
                for idx_dye=3:-1:1
                    switch idx_dye
                        case 1
                            stainingRatio=0.5;
                        case 2
                            stainingRatio=0.3;
                        case 3
                            stainingRatio=0.9;
                    end
                    vesicleDyeStaining_boolVect_act_dye_temp=stainingRatio>=rand(1, sum(vesIsPlaced_boolVect));
                    vesicleDyeStaining_boolVect_temp(idx_dye, :)=vesicleDyeStaining_boolVect_act_dye_temp;
                end
                % 50% of vesicles are stained with mCLING thar are labelled
                % with A-syn and with VAMP-2
                vesicleDyeStaining_boolVect_temp(1, :)=vesicleDyeStaining_boolVect_temp(1, :).*vesicleDyeStaining_boolVect_temp(2, :).*vesicleDyeStaining_boolVect_temp(3, :);
                vesicleDyeStaining_boolStruct(idxSynap).val=vesicleDyeStaining_boolVect_temp;
        end
        
    end
    
    vesicleDyeStaining_boolVect=[vesicleDyeStaining_boolStruct(:).val];
    
   %% vesicles in the solution
   
    ves_N_solution=params.sample.ves_rand_solution_N;  % number of vesicles
    ves_dim=params.sample.ves_rand_solution_area;      % "x" and "y" extension of area in which the vesicles are distributed
    ves_zmax=params.sample.ves_rand_zmax;    % "z" extension of area in which the vesicles are distributed
    
    solution.ves_pos=[];
    solution.ves_rad_max=[];
    solution.ves_rad=[];
    
    
    % loop throught the vesicles in the solution
    idx_ves=1;      % index of the actual vesicle
    actVesTries_N=0;      % number of tries given in placing the actual vesicle (the vesicles can not overlap)
    while idx_ves<=ves_N_solution
        actVesTries_N=actVesTries_N+1;  % one more try was given in placing the actual vesicle
        
        % place the vesicle randomly in a cube determined by the
        % "ves_dim" and "z_max" parameters, will discard it later, if it is outside
        % of the sphere
        actVesPos=[...
            (rand(1,1)-0.5)*ves_dim;...
            (rand(1,1)-0.5)*ves_dim;...
            (rand(1,1)-0.5)*ves_zmax];  % 1x(ves_fix_N/ves_N_ofves) array containing coordinates of vesicles
        
        % generate the radius for the actual vesicle
        actVesRad_max=R_mean_max+R_sig.*randn(1, 1);
        actVesRad=R_mean+R_sig.*randn(3, 1);
        
        % examine whether the actual vesicle is inside any of the
        % synaptosomes
        synaptosome_withinBool=false;
        for idxSynap=1:synaptosome_N
            if synaptosome_vesN(1, idxSynap)==0 % skip, if this synaptosome contains no vesicles
                synaptosome_withinBool=false;
            else
                synaptosome_withinBool=sum((actVesPos-synaptosome(idxSynap).pos).^2)<=(synaptosome(idxSynap).rad+actVesRad_max)^2;
            end
            if synaptosome_withinBool
                break;
            end
        end
        
        % examine whether the actual vesicle overlaps with any of the
        % previously placed vesicles in the solution
        if ~isempty(solution.ves_pos)      % if it is not the first vesicle in the solution
            vesDistances=sum((solution.ves_pos-repmat(actVesPos, [1, size(solution.ves_pos,2)])).^2,1);   % square of distances of the actual and the previously placed vesicles
            vesRadSums=solution.ves_rad_max+repmat(actVesRad_max, [1, size(solution.ves_rad_max,2)]);         % sum of radii of the actual and the previously placed vesicles
            vesDistances_RadSums_Bool=vesDistances>=vesRadSums.^2;      % examine whether actual vesicle overlaps with any other in the solution
            vesicle_distBool=prod(vesDistances_RadSums_Bool);       % true if the actual vesicle does not overlap with any other in the solution
        else        % if it is the first vesicle in the solution
            vesicle_distBool=true;
        end
        
        % if the actual vesicle is not inside any of the synaptosomes and does not
        % overlap with any of the previously placed vesicles, add it to
        % the parameters of the solution and leap to the next
        % vesicle
        if ~synaptosome_withinBool && vesicle_distBool
            idx_ves=idx_ves+1;
            actVesTries_N=0;
            solution.ves_pos=[solution.ves_pos, actVesPos];
            solution.ves_rad_max=[solution.ves_rad_max, actVesRad_max];
            solution.ves_rad=[solution.ves_rad, actVesRad];
        end
        
        % if too many tries was given in placing the vesicle, consider
        % not any more vesicle can be placed in this solution, leap
        % to the next synaptosome
        if actVesTries_N>100
            disp('Cannot place any more vesicle in the solution region.')
            disp([num2str(idx_ves-1),' vesicles were placed in this synaptosome.'])
            break;
        end
  
    end
    
    
    %% combined vesicles
    
    % concatenate the positions of the vesicles in different synaptosomes
    vesicle_coords.pos=[synaptosome(:).ves_pos, solution.ves_pos];  % 3x(ves_fix_N/ves_N_ofves) array containing coordinates of vesicles
    
    ves_N_final=size(vesicle_coords.pos, 2);
    
    % the total number of vesicels
    vesicle_coords.ves_N=ves_N_final;
    
    % concatenate the radii of the vesicles in different synaptosomes
    vesicle_coords.rad=[synaptosome(:).ves_rad, solution.ves_rad];
    
    % rotation angles of the ellipsoid
    %theta_rot_ellipsoids=acos(1-2*rand(N_ves,1));     % angle for the ellipsoid's rotation around the "y" axis
    vesicle_coords.theta=zeros(ves_N_final,1);      % angle for the ellipsoid's rotation around the "y" axis
    vesicle_coords.fi=2*pi*rand(ves_N_final,1);  % angle for the ellipsoid's rotation around the "z" axis
    %fi_rot_ellipsoids=zeros(N_ves,1);  % angle for the ellipsoid's rotation around the "z" axis
    
    % include the vesicle in the solution too
    vesicle_coords.vesicleDyeStaining_boolVect=[vesicleDyeStaining_boolVect, true(2, ves_N_solution)];
    
    
else
    error('Error in vesicles coordinates');
end

end

