%function [PSF_image, idx_pix_x, idx_pix_y]=m_gen_PSF_gauss( params )
function []=m_gen_PSF_gauss(  )

% do not comment this:
global Gaussian_beam_params_bool

%these are for testing purposes, otherwise should be commented
params.mol.pos=[00;00;2000];
params.aq.NA=1.4;                    % objective's numerical aperture
params.aq.n_immersion=1.518;          % refractive index of the immersion medium
params.aq.n_sample=1.331;          % refractive index of the immersion medium
params.dye.lambda=600;           % peak emission wavelength of the dye
params.aq.pixel_size=160;    % size of pixel in nm
params.aq.frame_size=64;    % number of pixels in one dimension
params.dye.em_N_ofphotonpersec=100000;
params.psf.colleff=0.4;
params.meas.astigmatic_bool=true;
Gaussian_beam_params_bool=false;
params.meas.tube_f=20;               % focal length of tube lens in cm
params.meas.magnification=100;       % magnification of the objective
params.meas.cylindrical_f=10;        % focal length of cylindrical lens in m
params.mol.ideal_PSF_bool=false;


% params.aq.focal_plane_dist=300;     % distance between the objective's focal point and frontal aperture in um

if params.mol.ideal_PSF_bool
    % place the molecule so it shall have the maximal possible PSF pixel values
    % ensure that the molecule is placed in the middle of a pixel
    if rem(params.aq.frame_size,2)==0
        params.mol.pos=[params.aq.pixel_size/2;params.aq.pixel_size/2;0];
    elseif rem(params.aq.frame_size,2)==1
        params.mol.pos=[0;0;0];
    else
        error('Something is wring with "frame_size" in "m_gen_trajs_to_seq.m"')
    end
end

pos=params.mol.pos;
pixel_size=params.aq.pixel_size*1E-9;	% size of pixel in m
frame_size=params.aq.frame_size;        % number of pixels in one dimension
em_N_ofphotonpersec=params.dye.em_N_ofphotonpersec;
colleff=params.psf.colleff;
PSF_N_ofphotonpersec=em_N_ofphotonpersec*colleff;

persistent w_0_x w_0_y z_R_x z_R_y z_shift_x z_shift_y    % permanent beam parameters

% calculating permanent beam parameters
if ~Gaussian_beam_params_bool   % if it has not calculated for the actual dye or the parameters has been changed (generate button has benn pressen once again)

    [w_0_x, w_0_y, z_R_x, z_R_y, z_shift_x, z_shift_y]=m_gen_PSF_Gaussian_beam_params(params);
    Gaussian_beam_params_bool=true;
end

% beam diameters
z_pos=pos(3)*1E-9;
% "z_shift_x" and "z_shift_y" has to be inverted so the "z_pos" results the
% right Gaussian PSF size on the detector
w_x=w_0_x*sqrt(1+(z_pos-(-z_shift_x))^2/z_R_x^2);      % beam diameter of the Gaussian beam in the "x-z" plane
w_y=w_0_y*sqrt(1+(z_pos-(-z_shift_y))^2/z_R_y^2);      % beam diameter of the Gaussian beam in the "y-z" plane

% input variables for pixelization, converting the diameters and positions
% to normalized coordinates
sigma_x=w_x/2/pixel_size;   % standard deviation of the PSF intensity distribution in "x" direction in normal coordinates
sigma_y=w_y/2/pixel_size;   % standard deviation of the PSF intensity distribution in "y" direction in normal coordinates
x=pos(1)*1E-9/pixel_size+frame_size*0.5; % "x" position in the image's coordinate system
y=pos(2)*1E-9/pixel_size+frame_size*0.5; % "y" position in the image's coordinate system


% upper border coordinates (and indices) of pixels
persistent posx posy
if isempty(posy) || length(posx(:,1))~=frame_size
    posy=repmat((1:frame_size),frame_size,1);
    posx=posy';
end

range_x=3*sigma_x;
range_y=3*sigma_y;

idx_pix_x=max(ceil(x-range_x),1):min(ceil(x+range_x),frame_size);
idx_pix_y=max(ceil(y-range_y),1):min(ceil(y+range_y),frame_size);

% calculating the PSF with "normalized" error functions
% http://mathworld.wolfram.com/NormalDistribution.html
PSF_image{1}=1/4.*PSF_N_ofphotonpersec*... 
    (erf((posx(idx_pix_x,idx_pix_y)-x)./(sqrt(2).*sigma_x))-erf((posx(idx_pix_x,idx_pix_y)-1-x)./(sqrt(2).*sigma_x))).*...
    (erf((posy(idx_pix_x,idx_pix_y)-y)./(sqrt(2).*sigma_y))-erf((posy(idx_pix_x,idx_pix_y)-1-y)./(sqrt(2).*sigma_y)));

figure()
mesh(PSF_image{1})

end