function [ F_aperture_func ] = m_gen_PSF_DFT( params, aperture_func )
% This function calculates the PSF in the image plane using 2D discrete
% Fourier-transform. Matrices required by the DFT are calculated only once
% for a given number of pixels taken into account (spatial frequencies are
% determined by the number of pixels taken into account).

% PARENT: m_gen_PSF_numerical_calc
% CHILD: -

global DFT_matrix_bool;

lambda=params.dye.lambda*1E-9;  % center wavelength of the emission in m

sampling_number_x=params.numcalc.sampling_number_x;
sampling_index_x=params.numcalc.sampling_index_x;
sampling_number_y=params.numcalc.sampling_number_y;
sampling_index_y=params.numcalc.sampling_index_y;
aperture_rad_tube=params.numcalc.aperture_rad_tube;
d_x_image=params.numcalc.d_x_image;
d_y_image=params.numcalc.d_y_image;

persistent d_x_ap ap_coords_x d_y_ap ap_coords_y    % coordiantes of the aperture's sampled points, needs to be calculated only once

global x_samp_tube_norm_int y_samp_tube_norm_int

if isempty(ap_coords_x)   % if the aperture's sampled points have yet to be calculated
    %d_x_ap=2*aperture_radius/sampling_number_x;
    %ap_coords_x=-aperture_radius+2*aperture_radius/sampling_number_x*(sampling_index_x-0.5);
    ap_coords_x=x_samp_tube_norm_int'*aperture_rad_tube;
    d_x_ap=(max(ap_coords_x)-min(ap_coords_x))/(sampling_number_x-1);
end

if isempty(ap_coords_y)   % if the aperture's sampled points have yet to be calculated
    %d_y_ap=2*aperture_radius/sampling_number_y;
    %ap_coords_y=-aperture_radius+2*aperture_radius/sampling_number_y*(sampling_index_y-0.5);
    ap_coords_y=y_samp_tube_norm_int*aperture_rad_tube;
    d_y_ap=(max(ap_coords_y)-min(ap_coords_y))/(sampling_number_y-1);
end

persistent DFT_matrix_pers calculated_matrices
% DFT_matrix_pers: cell storing the calculated DFT matrices
% N_pixel_calculated: array storing whether the DFT matrix have been calculated for the given number of pixels

% if the calculation is executed with the other dye or with changed
% parameters (generate button has been pressed one more time)
if ~DFT_matrix_bool
    DFT_matrix_pers=[];
    calculated_matrices=[];
end

% constant for spatial frequency calculation
persistent spat_freq_const

if isempty(spat_freq_const)
    spat_freq_const=2*pi/lambda;
end

DFT_idx_x=find(calculated_matrices==d_x_image+1i*sampling_number_x,1);  % index of the calculated DFT matrix in "DFT_matrix_pers"

if ~isempty(DFT_idx_x) % if the DFT matrix have yet to be calculated
    DFT_matrix_x=DFT_matrix_pers{DFT_idx_x};    % matrix for the actual DFT calculation
else
    im_coords=d_x_image*(sampling_number_x-1)*((sampling_index_x-1)/(sampling_number_x-1)-0.5);    % sampled point coordinates on the image plane
    % spatial frequencies in the image space coincide with the ones in the object space
    spat_freq=im_coords'*spat_freq_const;   % spatial frequencies for the Fourier-transform

    % the DFT martix have been calculated for a new number of pixels
    DFT_matrix_pers{end+1}=exp(-1i*spat_freq*ap_coords_x); % appending the new DFT matrix
    calculated_matrices=[calculated_matrices,d_x_image+1i*sampling_number_x];        % appending the new number of pixels for which the DFT matrix have benn calculated
    DFT_matrix_x=DFT_matrix_pers{end};    % matrix for the actual DFT calculation
end

DFT_idx_y=find(calculated_matrices==d_y_image+1i*sampling_number_y,1);  % index of the calculated DFT matrix in "DFT_matrix_pers"

if ~isempty(DFT_idx_y) % if the DFT matrix have yet to be calculated
    DFT_matrix_y=DFT_matrix_pers{DFT_idx_y};    % matrix for the actual DFT calculation
else
    im_coords=d_y_image*(sampling_number_y-1)*((sampling_index_y-1)/(sampling_number_y-1)-0.5);    % sampled point coordinates on the image plane
    % spatial frequencies in the image space coincide with the ones in the object space
    spat_freq=im_coords'*spat_freq_const;   % spatial frequencies for the Fourier-transform

    % the DFT martix have been calculated for a new number of pixels
    DFT_matrix_pers{end+1}=exp(-1i*spat_freq*ap_coords_y); % appending the new DFT matrix
    calculated_matrices=[calculated_matrices,d_y_image+1i*sampling_number_y];        % appending the new number of pixels for which the DFT matrix have benn calculated
    DFT_matrix_y=DFT_matrix_pers{end};    % matrix for the actual DFT calculation
end

F_aperture_func=DFT_matrix_x*aperture_func*DFT_matrix_y*d_x_ap*d_y_ap*spat_freq_const/(2*pi);    % 2D DFT calculation
% "DFT_matrix" is symmetric, no need for transposing
% the integration matrix ("int_mat") is taken into account in the "aperture_func"

% checking the equivalence of optical power transmitted through the aperture and hitting the detector
    %sum(sum(abs(aperture_func).^2))*d_x_ap*d_y_ap
    %sum(sum(abs(F_aperture_func).^2))*d_x_image*d_y_image
    
% checking the DFT matrix
    %figure()
    %mesh(abs(DFT_matrix_x))
    %figure()
    %mesh(real(DFT_matrix_x))
    %figure()
    %mesh(imag(DFT_matrix_x))

    %figure()
    %mesh(abs(DFT_matrix_y))
    %figure()
    %mesh(real(DFT_matrix_y))
    %figure()
    %mesh(imag(DFT_matrix_y))
    
end

