function [ ] = test_PSF_variance_defocus( )
%UNTITLED Summary of this function goes here
%   Detailed explanation goes here

function F=ray_angles(theta, h_ap)
    F=(h_ap-(d_i+delta_d_i)*tan(theta)-...
        d_g*n_i*sin(theta)/n_g./sqrt(1-(n_i*sin(theta)/n_g).^2)-...
        d_s*n_i*sin(theta)/n_s./sqrt(1-(n_i*sin(theta)/n_s).^2))*1E9;
end

% function for calculating the radial dependence of PSF during PSF
% size minimalization of PSF disrtorted by Gibson and Lanni aberration
function G=PSF_func(delta_d_i_iter, defocus_iter)
    G=abs((besselj(0,2*pi*2.5*0.5*ro_det_var*ro_tube_var')*(ro_tube_var.*apodizing_func_PSF_variance...           % Hankel transform of the approximated aperture function
        .*exp(1i*k*(d_s*sqrt(n_s^2-NA_LG^2*ro_tube_var.^2)+delta_d_i_iter*sqrt(n_i^2-NA_LG^2*ro_tube_var.^2)-defocus_iter*ro_tube_var.^2))))').^2;    % Gibson and Lanni aberration
end

% function for calculating the variance of PSF during PSF
% size minimalization of PSF disrtorted by Gibson and Lanni aberration
function I=PSF_defocus_variance(defocus_iter)
    delta_d_i_iter=delta_d_i;
    I=PSF_func(delta_d_i_iter, defocus_iter)...                         % PSF
      /(PSF_func(delta_d_i_iter, defocus_iter)*2*pi*ro_det_var)...     % normalization
        *(2*pi*ro_det_var.^3)*1E3;                         % multipliing with radius^2 for varianve ang integrating in polar coordiantes

end

lambda=600*1E-9;
k=2*pi/lambda;   % wave number in "1/m"
focal_plane_dist=300*1E-6;
NA=1.40;
d_s=5*1E-6;
n_s=1.331;
n_i=1.518;
d_i=focal_plane_dist;
n_g=n_i;
d_g=0;

% calculating the objective's aperture's radius from the NA, focal length
% and the refractive index of immersion medium
% the refractive index of the immersion medium and the cover slip are
% assumed to be equal
alpha_i=asin(NA/n_i);   %the maximal half-angle of the cone of light in the immersion medium

% if the angles are complex, e.g. wrong values were given either for the
% refractive indices or for the numerical aperture
if ~isreal(alpha_i)
    error('Wrong immersion medium RI for the given numerical aperture.')
end

aperture_rad_obj=focal_plane_dist*tan(alpha_i);  % radius of the objective's aperture in m

sampling_number_var=256;

ro_tube_var=(0.5:sampling_number_var-0.5)'/sampling_number_var;        % normalized radii values of the tube lens' aperture
ro_det_var=(0.5:sampling_number_var-0.5)'/sampling_number_var;         % normalized radii values of the imaged area

delta_d_i_0=-d_s*n_i/n_s;               % initial thickness change of the immersion medium
delta_d_i=delta_d_i_0;                  % thickness change of the immersion medium


% determining the maximal ray angles belonging to the total intrenal
% refraction in the immersion medium
switch max([n_s, n_g, n_i])
    case n_i    % if the immersion medium has the highest refractive index
        theta_i_max=asin(min([n_s, n_g, n_i])/n_i);
    case n_g    % if the cover slip has the highest refractive index
        if n_i>n_s
            theta_i_max=asin(min([n_s, n_g, n_i])/n_i);
        elseif n_i<=n_s
            theta_i_max=pi/2;
        end
    case n_s    % if the sample medium has the highest refractive index
        if n_i>n_g
            theta_i_max=asin(min([n_s, n_g, n_i])/n_i);
        elseif n_i<=n_g
            theta_i_max=pi/2;
        end
end

% initial guess for the ray angles belonging to the intersection border points
theta_edge_i_0=atan(abs(aperture_rad_obj)/(d_i+delta_d_i+d_s));     % initial guess for the ray angles, required for the iteration method

% approximated apodizing function, gives the "shape" of the aperture function
% did not want to calculate it here, did not want to run the expensive
% iteration for all ray angles here too and experienced as the the shape of
% the aperture function does not affects much the outcome variable value of
% minimalization of the PSF
apodizing_func_PSF_variance=cos(theta_edge_i_0*ro_tube_var).^1.5;           % this fits relatively well to the product of the apodizing function and the transmission coefficients (modified apodizing function)
%apodizing_func=1;

theta_max_ratio=0.95;   % starting maximum allowed ray angle and angle of total internal reflection ration
        
options = optimset('Display','off', 'FinDiffType', 'central');        % do not display iteration output
N_fsolve_loop=10;       % maximal number of loops for the iteration of "theta_i_ap_edge", inner loops, should be enough
fsolve_loop=1;       % variable for the "while" loop
while fsolve_loop<=N_fsolve_loop
    iter_success_bool=true;    % boolean for storing whether all the iterations were successful
    theta_edge_i_0_max=theta_i_max*theta_max_ratio;             % let this be the maximal initial ray angle, should be smaller than the angle of total internal refraction
    theta_edge_i_0_iter=theta_edge_i_0;                         % initial ray angle for the iteration
    theta_edge_i_0_iter(theta_edge_i_0_iter>theta_edge_i_0_max)=theta_edge_i_0_max;   % if any of the initial guess exceeds the maximal value...
    
    [theta_edge_i, ~, exitflag]=fsolve(@(theta) ray_angles(theta, abs(aperture_rad_obj)), theta_edge_i_0_iter, options);   % iteration for getting the ray angles
    % examining whether the iteration was successfull for a given ray
    if exitflag>0       % if the iteration was successful
    else                % if the iteration was not successful
        iter_success_bool=false;    % do not break out of the "while" loop
    end
    
    % if the iterations were successful for all angles...
    if iter_success_bool
        break;           % break out of the loop
    end
    % decrease the "theta_bord_i_0_max" and "theta_i_max" difference in
    % the next loop
    fsolve_loop=fsolve_loop+1;
    theta_max_ratio=1-(1-theta_max_ratio)/2;
end
    
if ~iter_success_bool
    error('no successfor ray angle')
end

NA_LG=n_i*sin(theta_edge_i);    % numerical aperture in Gibson and Lanni aberration
    
LG_0=d_s*n_s+delta_d_i*n_i;         % Lanni and Gibson aberration at the optical axis
LG_edge=(d_s*sqrt(n_s^2-NA_LG^2*1)+delta_d_i*sqrt(n_i^2-NA_LG^2*1));       % Lanni and Gibson aberration at the aperture edge
            
defocus_0=(LG_edge-LG_0)/8;     % let this be the initial value
    
[defocus,~,exitflag_fit]=fminsearch(@(defocus_iter) PSF_defocus_variance(defocus_iter), defocus_0);
    
if exitflag_fit>0       % if the iteration was successful
else                % if the iteration was not successful
    error('Could not minimalize the PSF variance. Please decrease the numerical aperture or the simple medium thickness.')
end
    

a=besselj(0,2*pi*2.5*0.5*ro_det_var*ro_tube_var')*(ro_tube_var.*apodizing_func_PSF_variance);     % aberration free PSF
b=PSF_func(delta_d_i,defocus);           % PSF after the iterations
c=PSF_func(delta_d_i,defocus_0);         % PSF with the initial values

figure()
plot(abs(a).^2)         % aberration free PSF

figure()
plot(b)                 % PSF after the iterations

figure()
plot(c)                 % PSF with the initial values

% values after and before the iterations
defocus
defocus_0

% PSF variance after and before the iterations
PSF_defocus_variance(defocus)
PSF_defocus_variance(defocus_0)


end

