function [ aperture_func ] = m_gen_PSF_aperture_func( params )
% This function calculates the amplitude masking of the aperture and the
% aberration of the wavefront at the aperture and return the complex
% amplitude of the electric field at the aperture's plane.

% PARENT: m_gen_PSF_numerical_calc
% CHILD: -

global aperture_func_bool;

n_i=params.aq.n_immersion;      % refractive index of immersion medium
n_g=n_i;                        % refractive index of cover slip
n_s=params.psf.n_sample;        % refractive index of sample

d_i=params.aq.focal_plane_dist*1E-6;	% thickness of the immersion medium and the cover slip under design conditions in m
d_g=0*1E-6;                             % thickness of the cover slip, it's taken into account in "d_i" in m
d_s=params.psf.sample_depth*1E-6;       % thickness of the sample layer in m
delta_d_i=-d_s*n_i/n_s;         % immersion medium thickness change in order to set the focus at the right sample depth for the paraxial rays in m

psf_type=params.psf.type;
lambda=params.dye.lambda*1E-9;  % emission wavelength in m
em_N_ofphotonpersec=params.dye.em_N_ofphotonpersec;
if strcmp(psf_type,'vectorial')
    anisotropy_bool=params.psf.anisotropy_bool;
    orientation=params.mol.init_orientation;
end

aperture_radius=params.numcalc.aperture_radius;     % radius of objective's aperture in m
sampling_number_x=params.numcalc.sampling_number_x;
sampling_number_y=params.numcalc.sampling_number_y;
sampling_index_x=params.numcalc.sampling_index_x;
sampling_index_y=params.numcalc.sampling_index_y;
spat_freq_correction=params.numcalc.spat_freq_correction;

persistent x y circ ro ro_2

% calculating the coordinates of the sampled points in a Descartes
% coordinate system normalized to the radius of the aperture
if isempty(x) || isempty(y)
    y_temp=(-1+(sampling_index_y-1)*2/(sampling_number_y-1))*(sampling_number_y-1)/sampling_number_y;  % vector of normalized y coordinates of sampled points

    x_temp=(-1+(sampling_index_x'-1)*2/(sampling_number_x-1))*(sampling_number_x-1)/sampling_number_x;  % vector of normalized x coordinates of sampled points

    x=repmat(x_temp,1,sampling_number_y);     % array containing the sampled points' normalized x coordinates

    y=repmat(y_temp,sampling_number_x,1);     % array containing the sampled points' normalized y coordinates
    
    ro_2=x.^2+y.^2;     % square of normalized radii of the sampled points
    ro=sqrt(ro_2);      % normalized radii of the sampled points

end

if isempty(circ)
    circ=zeros(sampling_number_x,sampling_number_y);        % circular function of the aperture
    circ(ro<=1)=1;
end

%width_x=aperture_radius/10;
%width_y=aperture_radius/10;
%aperture_func{1}=exp(-((sampling_index_y'/(sampling_number_y/2)-1)*aperture_radius).^2/(width_x^2))*exp(-((sampling_index_x/(sampling_number_x/2)-1)*aperture_radius).^2/(width_y^2)).*circ;      % calculating the electric field in the aperture
aperture_func{1}=ones(sampling_number_x, sampling_number_y).*circ;

end

