function [ ] = test_TestSTORM_PSF(  )

% This function is suitable for testing the PSFs generated by TestSTORM.

filename=mfilename('fullpath');
[m_gen_path] = fileparts(filename);

addpath([m_gen_path,filesep,'functions'])

%% Parameters to set:
% Coordinates of the fluorophores [nm]:
xCoords_mol=[0];
yCoords_mol=[0];
zCoords_mol=[0];

% Parameters for 3D astigmatic simulation
params.meas.astigmatic_bool=false;
params.meas.tube_f=20;      %in "cm"
params.meas.magnification=100;
params.meas.cylindrical_f=10;   % in "m"

% Choose a PSF model
%params.psf.type='gaussian';
params.psf.type='scalar';
%params.psf.type='vectorial';
% Parameters for the polarization sensitive simulation
params.meas.polarization_bool=false;

if params.meas.polarization_bool && ~strcmp(params.psf.type, 'vectorial')
   error('Polarization sensitive simulation can only be performed with the vectorial PSF model.') 
end

%Parameters for the Gaussian PSf model
params.psf.colleff=0.3;

% Parameters for the scalar or for the vectorial PSF model
params.psf.focal_plane_dist=300;       % distance between the focal plane and the objective's aperture in um
params.psf.sample_depth=5;
params.psf.an_cone_angle=5;
params.psf.el_field_dir=[1;1i;0];
params.psf.an_tau_ratio=Inf;
params.psf.anisotropy_bool=false;
params.psf.readApertureFuncBool=false;

% Orientation of the fluorphores, for the vectorial PSF model
params.mol.orientation(1)=45*pi/180;       % fi (azimuth) angle of orientation of the dipole
params.mol.orientation(2)=90*pi/180;       % theta (polar) angle of orientation of the dipole

% dye parameters
params.dye.em_N_ofphotonpersec=5000;
params.dye.lambda=665;

% Acquisition parameters
params.aq.frame_size=10;
params.aq.pixel_size=160;
params.aq.NA=1.4;
params.aq.n_immersion=1.52;
params.aq.n_sample=1.33;

params.mol.ideal_PSF_bool=false;


%% frame (single) generation
pos=[...
    xCoords_mol;...
    yCoords_mol;...
    zCoords_mol];

NBlinking=size(pos,2);

frameSize=params.aq.frame_size;

Frame=zeros(frameSize,frameSize);

global Gaussian_beam_params_bool;
global DFT_matrix_bool;
global aperture_func_bool;
global numerical_calc_cont_bool

% booleans for deciding whether the persistent variables should be calculated (or recalculated)
Gaussian_beam_params_bool=false;
DFT_matrix_bool=false;
aperture_func_bool=false;
numerical_calc_cont_bool=false;

for idxBlinking=1:NBlinking

    params.mol.pos=pos(:,idxBlinking);
    
    if strcmp(params.psf.type,'vectorial') && params.psf.readApertureFuncBool
    % if the electrical field at the aperture should be read from an
    % external simulation
        params.lumerical_monitorDistance=40*1e-9;
        params.lumerical_dipoleDepth=30*sin(20*pi/180)*1e-9;
        
        params.psf.sample_depth=params.lumerical_dipoleDepth;
        
        [PSF_image, idx_pix_x, idx_pix_y]=m_gen_PSF_calc( params, true );
    else 
    % generating the PSFs completely with TestSTORM
        [PSF_image, idx_pix_x, idx_pix_y]=m_gen_PSF_calc( params, true );
    end
    
    % adding the pixelized PSF to the stack
    Frame(idx_pix_x, idx_pix_y)=Frame(idx_pix_x, idx_pix_y)+PSF_image{1};
    
    if strcmp(params.psf.type,'vectorial') && params.meas.polarization_bool
        Frame(idx_pix_x, idx_pix_y)=Frame(idx_pix_x, idx_pix_y)+PSF_image{2};
    end
    
end

% clearing the persistent variables used for PSF generation
clear m_gen_PSF_gauss % it must be cleared as frame size can change in the next simulation
% no need to clear them, just for safety reason:
clear m_gen_PSF_numerical_calc
clear m_gen_PSF_aperture_func
clear m_gen_PSF_DFT
clear m_gen_PSF_pixelise


%figure()
%mesh(Frame)

%% displaying results
figure()
image(Frame(:,:,1)'/max(max(Frame(:,:,1)))*64)
axis square
title('Pixelized PSF')

end

